@@ -247,6 +247,7 @@ class WarningHunter(object):
                                              module)
 
         def _add_declaration(name, namespace):
+            assert name
             names = [n for n in namespace if n is not None]
             if names:
                 name = '::'.join(names) + '::' + name
@@ -399,7 +400,8 @@ class WarningHunter(object):
                     ast_seq.append(node.body)
                 elif isinstance(node, ast.Class) and node.body is not None:
                     _add_declaration(node.name, node.namespace)
-                    _add_template_use('', node.bases, node.namespace)
+                    namespace = namespace_stack + node.namespace
+                    _add_template_use('', node.bases, namespace)
                     ast_seq.append(node.body)
                 elif isinstance(node, ast.Using):
                     if node.names[0].name == 'namespace':
@@ -432,9 +434,25 @@ class WarningHunter(object):
                        "but already #included in '{}'".format(node.name, name))
                 self._add_warning(msg, node)
 
+    def _find_incorrect_case(self, included_files):
+        for (filename, node_and_module) in included_files.items():
+            base_name = os.path.basename(filename)
+            try:
+                candidates = os.listdir(os.path.dirname(filename))
+            except OSError:
+                continue
+
+            correct_filename = get_correct_include_filename(base_name,
+                                                            candidates)
+            if correct_filename:
+                self._add_warning(
+                    "'{}' should be '{}'".format(base_name, correct_filename),
+                    node_and_module[0])
+
     def _find_header_warnings(self):
         included_files, forward_declarations = self._read_and_parse_includes()
         self._find_unused_warnings(included_files, forward_declarations)
+        self._find_incorrect_case(included_files)
 
     def _find_public_function_warnings(self, node, name, primary_header,
                                        all_headers):
@@ -518,6 +536,8 @@ class WarningHunter(object):
 
     def _find_source_warnings(self):
         included_files, forward_declarations = self._read_and_parse_includes()
+        self._find_incorrect_case(included_files)
+
         for node in forward_declarations.values():
             # TODO(nnorwitz): This really isn't a problem, but might
             # be something to warn against. I expect this will either
@@ -558,6 +578,14 @@ def get_line_number(metrics_instance, node):
     return metrics_instance.get_line_number(node.start)
 
 
+def get_correct_include_filename(filename, candidate_filenames):
+    if filename not in candidate_filenames:
+        for candidate in candidate_filenames:
+            if filename.lower() == candidate.lower():
+                return candidate
+    return None
+
+
 def run(filename, source, entire_ast, include_paths, quiet):
     hunter = WarningHunter(filename, source, entire_ast,
                            include_paths=include_paths,
